@@ -1,5 +1,7 @@
-#ifndef PQCLEAN_MCELIECE6960119F_CLEAN_PARAMS_H
-#define PQCLEAN_MCELIECE6960119F_CLEAN_PARAMS_H
+#ifndef PARAMS_H
+#define PARAMS_H
+
+#include "namespace.h"
 
 #define GFBITS 13
 #define SYS_N 6960
@@ -12,10 +14,8 @@
 #define PK_NCOLS (SYS_N - PK_NROWS)
 #define PK_ROW_BYTES ((PK_NCOLS + 7)/8)
 
-#define SK_BYTES (SYS_N/8 + IRR_BYTES + COND_BYTES)
 #define SYND_BYTES ((PK_NROWS + 7)/8)
 
 #define GFMASK ((1 << GFBITS) - 1)
 
 #endif
-
