@@ -1,5 +1,3 @@
-#include "inner.h"
-
 /*
  * Falcon signature generation.
  *
@@ -31,6 +29,7 @@
  * @author   Thomas Pornin <thomas.pornin@nccgroup.com>
  */
 
+#include "inner.h"
 
 /* =================================================================== */
 
@@ -154,7 +153,7 @@ ffLDL_fft(fpr *tree, const fpr *g00,
  * sigma / sqrt(x).
  */
 static void
-ffLDL_binary_normalize(fpr *tree, unsigned logn) {
+ffLDL_binary_normalize(fpr *tree, unsigned orig_logn, unsigned logn) {
     /*
      * TODO: make an iterative version.
      */
@@ -167,11 +166,11 @@ ffLDL_binary_normalize(fpr *tree, unsigned logn) {
          * the value mandated by the specification: this
          * saves a division both here and in the sampler.
          */
-        tree[0] = fpr_mul(fpr_sqrt(tree[0]), fpr_inv_sigma);
+        tree[0] = fpr_mul(fpr_sqrt(tree[0]), fpr_inv_sigma[orig_logn]);
     } else {
-        ffLDL_binary_normalize(tree + n, logn - 1);
+        ffLDL_binary_normalize(tree + n, orig_logn, logn - 1);
         ffLDL_binary_normalize(tree + n + ffLDL_treesize(logn - 1),
-                               logn - 1);
+                               orig_logn, logn - 1);
     }
 }
 
@@ -267,7 +266,7 @@ PQCLEAN_FALCON512_CLEAN_expand_privkey(fpr *expanded_key,
     PQCLEAN_FALCON512_CLEAN_poly_neg(rF, logn);
 
     /*
-     * The Gram matrix is G = B x B*. Formulas are:
+     * The Gram matrix is G = B·B*. Formulas are:
      *   g00 = b00*adj(b00) + b01*adj(b01)
      *   g01 = b00*adj(b10) + b01*adj(b11)
      *   g10 = b10*adj(b00) + b11*adj(b01)
@@ -307,7 +306,7 @@ PQCLEAN_FALCON512_CLEAN_expand_privkey(fpr *expanded_key,
     /*
      * Normalize tree.
      */
-    ffLDL_binary_normalize(tree, logn);
+    ffLDL_binary_normalize(tree, logn, logn);
 }
 
 typedef int (*samplerZ)(void *ctx, fpr mu, fpr sigma);
@@ -322,7 +321,7 @@ static void
 ffSampling_fft_dyntree(samplerZ samp, void *samp_ctx,
                        fpr *t0, fpr *t1,
                        fpr *g00, fpr *g01, fpr *g11,
-                       unsigned logn, fpr *tmp) {
+                       unsigned orig_logn, unsigned logn, fpr *tmp) {
     size_t n, hn;
     fpr *z0, *z1;
 
@@ -335,7 +334,7 @@ ffSampling_fft_dyntree(samplerZ samp, void *samp_ctx,
         fpr leaf;
 
         leaf = g00[0];
-        leaf = fpr_mul(fpr_sqrt(leaf), fpr_inv_sigma);
+        leaf = fpr_mul(fpr_sqrt(leaf), fpr_inv_sigma[orig_logn]);
         t0[0] = fpr_of(samp(samp_ctx, t0[0], leaf));
         t1[0] = fpr_of(samp(samp_ctx, t1[0], leaf));
         return;
@@ -378,7 +377,7 @@ ffSampling_fft_dyntree(samplerZ samp, void *samp_ctx,
     z1 = tmp + n;
     PQCLEAN_FALCON512_CLEAN_poly_split_fft(z1, z1 + hn, t1, logn);
     ffSampling_fft_dyntree(samp, samp_ctx, z1, z1 + hn,
-                           g11, g11 + hn, g01 + hn, logn - 1, z1 + n);
+                           g11, g11 + hn, g01 + hn, orig_logn, logn - 1, z1 + n);
     PQCLEAN_FALCON512_CLEAN_poly_merge_fft(tmp + (n << 1), z1, z1 + hn, logn);
 
     /*
@@ -401,7 +400,7 @@ ffSampling_fft_dyntree(samplerZ samp, void *samp_ctx,
     z0 = tmp;
     PQCLEAN_FALCON512_CLEAN_poly_split_fft(z0, z0 + hn, t0, logn);
     ffSampling_fft_dyntree(samp, samp_ctx, z0, z0 + hn,
-                           g00, g00 + hn, g01, logn - 1, z0 + n);
+                           g00, g00 + hn, g01, orig_logn, logn - 1, z0 + n);
     PQCLEAN_FALCON512_CLEAN_poly_merge_fft(t0, z0, z0 + hn, logn);
 }
 
@@ -788,7 +787,7 @@ do_sign_dyn(samplerZ samp, void *samp_ctx, int16_t *s2,
     PQCLEAN_FALCON512_CLEAN_poly_neg(b11, logn);
 
     /*
-     * Compute the Gram matrix G = B x B*. Formulas are:
+     * Compute the Gram matrix G = B·B*. Formulas are:
      *   g00 = b00*adj(b00) + b01*adj(b01)
      *   g01 = b00*adj(b10) + b01*adj(b11)
      *   g10 = b10*adj(b00) + b11*adj(b01)
@@ -872,7 +871,7 @@ do_sign_dyn(samplerZ samp, void *samp_ctx, int16_t *s2,
      * Apply sampling; result is written over (t0,t1).
      */
     ffSampling_fft_dyntree(samp, samp_ctx,
-                           t0, t1, g00, g01, g11, logn, t1 + n);
+                           t0, t1, g00, g01, g11, logn, logn, t1 + n);
 
     /*
      * We arrange the layout back to:
@@ -1082,8 +1081,8 @@ BerExp(prng *p, fpr x, fpr ccs) {
 int
 PQCLEAN_FALCON512_CLEAN_sampler(void *ctx, fpr mu, fpr isigma) {
     sampler_context *spc;
-    int s, z0, z, b;
-    fpr r, dss, ccs, x;
+    int s;
+    fpr r, dss, ccs;
 
     spc = ctx;
 
@@ -1108,6 +1107,9 @@ PQCLEAN_FALCON512_CLEAN_sampler(void *ctx, fpr mu, fpr isigma) {
      * We now need to sample on center r.
      */
     for (;;) {
+        int z0, z, b;
+        fpr x;
+
         /*
          * Sample z for a Gaussian distribution. Then get a
          * random bit b to turn the sampling into a bimodal
@@ -1187,11 +1189,7 @@ PQCLEAN_FALCON512_CLEAN_sign_tree(int16_t *sig, inner_shake256_context *rng,
          * Normal sampling. We use a fast PRNG seeded from our
          * SHAKE context ('rng').
          */
-        if (logn == 10) {
-            spc.sigma_min = fpr_sigma_min_10;
-        } else {
-            spc.sigma_min = fpr_sigma_min_9;
-        }
+        spc.sigma_min = fpr_sigma_min[logn];
         PQCLEAN_FALCON512_CLEAN_prng_init(&spc.p, rng);
         samp = PQCLEAN_FALCON512_CLEAN_sampler;
         samp_ctx = &spc;
@@ -1234,11 +1232,7 @@ PQCLEAN_FALCON512_CLEAN_sign_dyn(int16_t *sig, inner_shake256_context *rng,
          * Normal sampling. We use a fast PRNG seeded from our
          * SHAKE context ('rng').
          */
-        if (logn == 10) {
-            spc.sigma_min = fpr_sigma_min_10;
-        } else {
-            spc.sigma_min = fpr_sigma_min_9;
-        }
+        spc.sigma_min = fpr_sigma_min[logn];
         PQCLEAN_FALCON512_CLEAN_prng_init(&spc.p, rng);
         samp = PQCLEAN_FALCON512_CLEAN_sampler;
         samp_ctx = &spc;
